﻿/*
Populate_DataSets.jsx
by Chip Brown

Illustrator script to create DataSets from a tab delimited text field.

How To Use:
You must first select a text field with tab delimited data. The first row is used as the variable 
names, the remaining rows are used to create DataSets in the Variables pallete. If a variable doesn't
exist one is created for it, along with a stand-in text field. Missing variables are created as Text Variables.

Formating the Variables
	Textual - plain strings
	Visibility - rebresented by either a 1 or "true" if true, otherwise they're false
	LinkedFile - the file path relative to the document using "/" sintax 
			   (ex: foo.jpg is in the same folder, while imgs/foo.jpg is stored in an folder named 
			   'imgs' located in the same directory as the document.)
			   If the image has a visibility variable assigned to it as well, it will auto hide/show
			   based on whether or not the supplied image exists.

If you wish to deliniate the elements with something other than a tab (say a coma) just change 
the "delimiter" variable at the top of the script.

By default the existing DataSets are removed before adding the new ones, this can be altered by 
setting "clearOldSets" to false.

Known Issues:
	- Visibiliy works for everything but top level layers. Sub-layers, shapes, groups, etc. all work as expected.
	- Graph variables are not supported
*/

// define the character to use as a delimiter, \t is a Tab
var delimiter = "\t";
// remove existing DataSets before creating new one's
var clearOldSets = true;


//$.write("\r>>> Populate DataSet\r\r");

var docRef = activeDocument;
var success = true;
var placeholders = [];
var sourceText = getSourceText( activeDocument );


if ( sourceText==undefined ){
	errorMsg();
} else {
	populateDataSets( sourceText );
	if ( success ) alert("DataSet's Created");
}

/**
*	Returns the text contents of the current text field, or nothing if not a text field
*/
function getSourceText ( aDoc )
{	
	var sel = activeDocument.selection[0];
	if ( sel!=undefined ){
		// Entire Text Field Selected
		return sel.contents;
	} else {
		sel = activeDocument.selection.parent;
		if ( sel!=undefined && sel.typename=="Story" ){
			// Selection inside text field
			return sel.textFrames[0].contents;
		}
	}
}

/**
*	Core Function, loops through all rows of the text creating DataSets
*/
function populateDataSets ( text )
{
	var rows = text.split("\r");
	var keys = rows.shift().split(delimiter);
	var tempItems = [];
	
	if ( rows.length==0 ) return errorMsg();
		
	if ( clearOldSets ) docRef.dataSets.removeAll();
	
	// Record DataSet Values
	for ( var r in rows ){
		var fields = rows[r].split(delimiter);
		
		for ( var k in keys ){
			var key = keys[k];
			var value = fields[k];
			
			updateVarItems(key,value);
			
			//$.write("\t"+key+":"+value);
		}
		redraw();
		
		var newDataSet = docRef.dataSets.add();
		newDataSet.update();
		newDataSet.display();
		newDataSet.name="set_"+(Number(r)+1);
	}
	
	// Display the first DataSet
	docRef.dataSets[0].display();
}


/**
*	Sets the value of a variable for all occurances on the page
*/
function updateVarItems ( varName, value )
{
	//$.write( "\rupdate var - "+varName+" = "+value);
	
	var varItem = getVariable(varName);
	
	//$.write(varItem+" "+varItem.pageItems);
	
	var pgItmCount = varItem.pageItems.length;
	var updateCount = 0;
	for ( var i=0; i<pgItmCount; i++ ){
		// BUG FIX: We need a try-catch to account for variables whos pageItems are broken
		//try {
			var item = varItem.pageItems[i];
			switch ( varItem.kind ){
				case VariableKind.GRAPH:
					
					break;
				case VariableKind.IMAGE:
					if ( value==undefined || value=="" ){
						var img = File();
					} else {
						var relPath = docRef.path.relativeURI
						var img = File(relPath+"/"+value);
					}
					if ( img.exists ){
						item.file = img;
						item.hidden = false;
					} else {
						item.hidden = true;
					}
					break;
				case VariableKind.TEXTUAL:
					item.contents = (value==undefined ? "" : value );
					break;
				case VariableKind.Unknown:
					
					break;
				case VariableKind.VISIBILITY:
					//Assumes GroupItem is the target
					item.hidden = (value==undefined ? false : !(value==1 || value.toLowerCase()=='true') );
					break;
			}
			updateCount++;
		//} catch ( err ){
			//$.write("problem with item: "+err);
		//}
	}
	if ( updateCount==0 ){
		// There was an issue with updating the pageItems, recreate the variable
		varItem.remove();
		varItem = getVariable(varName, value);
	}
}

/**
*	BUG FIX: This function is used instead of Variables.getByName to avoid an error if the variable is not defined
*/
function getVariable ( varName, contents )
{
	// Look for variable and return it if it exists
	var count = docRef.variables.length;
	for ( var i=0; i<count; i++ ){
		var item = docRef.variables[i];
		if ( item.name == varName ) return item;
	}
	// Create a  variable & placeholder text field if it doesn't exist
	var varItem = docRef.variables.add();
	varItem.kind = VariableKind.TEXTUAL;
	varItem.name = varName;
	createTextFrame(varItem, contents);
	return varItem;
}

/**
*	Creates a simple text field to hold the variable, because you have to have content to create a DataSet
*/
function createTextFrame ( varItem, contents )
{
	var txtFrame = docRef.textFrames.add();
	txtFrame.contents = ( contents!=undefined ? contents : varItem.name );
	txtFrame.top = docRef.height-((txtFrame.height+5)*placeholders.length);
	txtFrame.left = 5;
	txtFrame.contentVariable = varItem;
	placeholders.push(txtFrame);
	redraw();
}

function errorMsg ()
{
	success = false;
	
	var msg = "You must select a tab-delimited text field as the source for the new Data Sets.";
	msg += "\rThe first line is used for the variable names.";
	msg += "\rThe remaining lines are individual data sets.";
	msg += "\r\rExample:\r";
	msg += "\rname \tage\tsex";
	msg += "\rBob    \t22\tmale";
	msg += "\rJill      \t28\tfemale";
	msg += "\rPat    \t32\twho know's?";
	alert( msg );
}

